class ViberRequest(object):
	def __init__(self, event_type=None):
		self._event_type = event_type
		self._timestamp = None

	def from_dict(self, request_dict):
		self._timestamp = request_dict['timestamp']
		if self._event_type is None:
			self._event_type = request_dict['event']
		return self

	@property
	def event_type(self):
		return self._event_type

	@property
	def timestamp(self):
		return self._timestamp



from ..version import __version__

VIBER_BOT_API_URL = "https://chatapi.viber.com/pa"
VIBER_BOT_USER_AGENT = "ViberBot-Python/" + __version__


class BOT_API_ENDPOINT(object):
	SET_WEBHOOK = 'set_webhook'
	GET_ACCOUNT_INFO = 'get_account_info'
	SEND_MESSAGE = 'send_message'
	GET_ONLINE = 'get_online'
	GET_USER_DETAILS = 'get_user_details'
	POST = 'post'



from viberbot_prox.api.messages.data_types.contact import Contact
from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class ContactMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, contact=None, min_api_version=None):
		super(ContactMessage, self).__init__(MessageType.CONTACT, tracking_data, keyboard, min_api_version)
		self._contact = contact

	def to_dict(self):
		message_data = super(ContactMessage, self).to_dict()
		if self._contact is not None:
			message_data['contact'] = self._contact.to_dict()
		return message_data

	def from_dict(self, message_data):
		super(ContactMessage, self).from_dict(message_data)
		if 'contact' in message_data:
			self._contact = Contact().from_dict(message_data['contact'])
		return self

	@property
	def contact(self):
		return self._contact

	def validate(self):
		return super(ContactMessage, self).validate() \
				and self._contact is not None \
				and self._contact.name is not None \
				and self._contact.phone_number is not None



from viberbot_prox.api.messages.contact_message import ContactMessage
from viberbot_prox.api.messages.file_message import FileMessage
from viberbot_prox.api.messages.picture_message import PictureMessage
from viberbot_prox.api.messages.sticker_message import StickerMessage
from viberbot_prox.api.messages.url_message import URLMessage
from viberbot_prox.api.messages.video_message import VideoMessage
from viberbot_prox.api.messages.message_type import MessageType
from viberbot_prox.api.messages.text_message import TextMessage
from viberbot_prox.api.messages.location_message import LocationMessage
from viberbot_prox.api.messages.rich_media_message import RichMediaMessage
from viberbot_prox.api.messages.keyboard_message import KeyboardMessage

MESSAGE_TYPE_TO_CLASS = {
	MessageType.URL: URLMessage,
	MessageType.LOCATION: LocationMessage,
	MessageType.PICTURE: PictureMessage,
	MessageType.CONTACT: ContactMessage,
	MessageType.FILE: FileMessage,
	MessageType.TEXT: TextMessage,
	MessageType.VIDEO: VideoMessage,
	MessageType.STICKER: StickerMessage,
	MessageType.RICH_MEDIA: RichMediaMessage,
	MessageType.KEYBOARD: KeyboardMessage
}


def get_message(message_dict):
	if 'type' not in message_dict:
		raise Exception("message data doesn't contain a type")

	if message_dict['type'] not in MESSAGE_TYPE_TO_CLASS:
		raise Exception(u"message type '{0}' is not supported".format(message_dict['type']))

	return MESSAGE_TYPE_TO_CLASS[message_dict['type']]().from_dict(message_dict)


__all__ = [
	'TextMessage', 'ContactMessage', 'FileMessage', 'LocationMessage',
	'PictureMessage', 'StickerMessage', 'URLMessage', 'VideoMessage',
	'RichMediaMessage', 'MessageType', 'KeyboardMessage']



from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class TextMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, text=None, min_api_version=None):
		super(TextMessage, self).__init__(MessageType.TEXT, tracking_data, keyboard, min_api_version)
		self._text = text

	def to_dict(self):
		message_data = super(TextMessage, self).to_dict()
		message_data['text'] = self._text
		return message_data

	def from_dict(self, message_data):
		super(TextMessage, self).from_dict(message_data)
		if 'text' in message_data:
			self._text = message_data['text']
		return self

	def validate(self):
		return super(TextMessage, self).validate() \
				and self._text is not None

	@property
	def text(self):
		return self._text



from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class VideoMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, media=None, thumbnail=None, size=None, text=None, duration=None, min_api_version=None):
		super(VideoMessage, self).__init__(MessageType.VIDEO, tracking_data, keyboard, min_api_version)
		self._media = media
		self._thumbnail = thumbnail
		self._size = size
		self._duration = duration
		self._text = text

	def to_dict(self):
		message_data = super(VideoMessage, self).to_dict()
		message_data['media'] = self._media
		message_data['thumbnail'] = self._thumbnail
		message_data['size'] = self._size
		message_data['duration'] = self._duration
		message_data['text'] = self._text
		return message_data

	def from_dict(self, message_data):
		super(VideoMessage, self).from_dict(message_data)
		if 'media' in message_data:
			self._media = message_data['media']
		if 'thumbnail' in message_data:
			self._thumbnail = message_data['thumbnail']
		if 'size' in message_data:
			self._size = message_data['size']
		if 'duration' in message_data:
			self._duration = message_data['duration']
		if 'text' in message_data:
			self._text = message_data['text']
		return self

	def validate(self):
		return super(VideoMessage, self).validate() \
				and self._media is not None \
				and self._size is not None

	@property
	def media(self):
		return self._media

	@property
	def thumbnail(self):
		return self._thumbnail

	@property
	def size(self):
		return self._size

	@property
	def duration(self):
		return self._duration

	@property
	def text(self):
		return self._text



from viberbot_prox.api.event_type import EventType
from viberbot_prox.api.viber_requests.viber_request import ViberRequest


class ViberUnsubscribedRequest(ViberRequest):
	def __init__(self):
		super(ViberUnsubscribedRequest, self).__init__(EventType.UNSUBSCRIBED)
		self._user_id = None

	def from_dict(self, request_dict):
		super(ViberUnsubscribedRequest, self).from_dict(request_dict)
		self._user_id = request_dict['user_id']
		return self

	@property
	def user_id(self):
		return self._user_id



from viberbot_prox.api import messages
from viberbot_prox.api.event_type import EventType
from viberbot_prox.api.user_profile import UserProfile
from viberbot_prox.api.viber_requests.viber_request import ViberRequest


class ViberMessageRequest(ViberRequest):
	def __init__(self):
		super(ViberMessageRequest, self).__init__(EventType.MESSAGE)
		self._message = None
		self._sender = None
		self._message_token = None
		self._chat_id = None
		self._reply_type = None
		self._silent = None

	def from_dict(self, request_dict):
		super(ViberMessageRequest, self).from_dict(request_dict)
		self._message = messages.get_message(request_dict['message'])
		self._sender = UserProfile().from_dict(request_dict['sender'])
		self._message_token = request_dict['message_token']
		self._silent = request_dict.get('silent', None)
		self._reply_type = request_dict.get('reply_type', None)
		self._chat_id = request_dict.get('chat_id', None)
		return self

	@property
	def message(self):
		return self._message

	@property
	def sender(self):
		return self._sender

	@property
	def message_token(self):
		return self._message_token

	@property
	def chat_id(self):
		return self._chat_id

	@property
	def reply_type(self):
		return self._reply_type

	@property
	def silent(self):
		return self._silent



from viberbot_prox.api.event_type import EventType
from viberbot_prox.api.user_profile import UserProfile
from viberbot_prox.api.viber_requests.viber_request import ViberRequest


class ViberSubscribedRequest(ViberRequest):
	def __init__(self):
		super(ViberSubscribedRequest, self).__init__(EventType.SUBSCRIBED)
		self._user = None
		self._api_version = None

	def from_dict(self, request_dict):
		super(ViberSubscribedRequest, self).from_dict(request_dict)
		self._user = UserProfile().from_dict(request_dict['user'])
		if 'api_version' in request_dict:
			self._api_version = request_dict['api_version']
		return self

	@property
	def user(self):
		return self._user

	@property
	def api_version(self):
		return self._api_version



from viberbot_prox.api.messages.message import Message


class KeyboardMessage(Message):
	def __init__(self, tracking_data=None, keyboard=None, min_api_version=None):
		super(KeyboardMessage, self).__init__(tracking_data, keyboard, min_api_version)

	def to_dict(self):
		return super(KeyboardMessage, self).to_dict()

	def from_dict(self, message_data):
		super(KeyboardMessage, self).from_dict(message_data)
		return self

	def validate(self):
		return self._keyboard is not None



from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class RichMediaMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, rich_media=None, min_api_version=None, alt_text=None):
		super(RichMediaMessage, self).__init__(MessageType.RICH_MEDIA, tracking_data, keyboard, min_api_version)
		self._rich_media = rich_media
		self._alt_text = alt_text

	def to_dict(self):
		message_data = super(RichMediaMessage, self).to_dict()
		message_data['rich_media'] = self._rich_media
		message_data['alt_text'] = self._alt_text
		return message_data

	def from_dict(self, message_data):
		super(RichMediaMessage, self).from_dict(message_data)
		if 'rich_media' in message_data:
			self._rich_media = message_data['rich_media']
		if 'alt_text' in message_data:
			self._alt_text = message_data['alt_text']
		return self

	def validate(self):
		return super(RichMediaMessage, self).validate() \
				and self._rich_media is not None

	@property
	def rich_media(self):
		return self._rich_media

	@property
	def alt_text(self):
		return self._alt_text



from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class StickerMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, sticker_id=None, min_api_version=None):
		super(StickerMessage, self).__init__(MessageType.STICKER, tracking_data, keyboard, min_api_version)
		self._sticker_id = sticker_id

	def to_dict(self):
		message_data = super(StickerMessage, self).to_dict()
		message_data['sticker_id'] = self._sticker_id
		return message_data

	def from_dict(self, message_data):
		super(StickerMessage, self).from_dict(message_data)
		if 'sticker_id' in message_data:
			self._sticker_id = message_data['sticker_id']
		return self

	@property
	def sticker_id(self):
		return self._sticker_id

	def validate(self):
		return super(StickerMessage, self).validate() \
				and self._sticker_id is not None



class Contact(object):
	def __init__(self, name=None, phone_number=None, avatar=None):
		self._name = name
		self._phone_number = phone_number
		self._avatar = avatar

	def from_dict(self, contact):
		if 'name' in contact:
			self._name = contact['name']
		if 'phone_number' in contact:
			self._phone_number = contact['phone_number']
		if 'avatar' in contact:
			self._avatar = contact['avatar']
		return self

	def to_dict(self):
		return {
			'name': self._name,
			'phone_number': self._phone_number,
			'avatar': self._avatar
		}

	@property
	def name(self):
		return self._name

	@property
	def phone_number(self):
		return self._phone_number

	def __eq__(self, other):
		return self._name == other.name and self._phone_number == other.phone_number



import warnings
from viberbot_prox.api.event_type import EventType
from viberbot_prox.api.viber_requests.viber_request import ViberRequest


class ViberFailedRequest(ViberRequest):
	def __init__(self):
		super(ViberFailedRequest, self).__init__(EventType.FAILED)
		self._message_token = None
		self._user_id = None
		self._desc = None

	def from_dict(self, request_dict):
		super(ViberFailedRequest, self).from_dict(request_dict)
		self._message_token = request_dict['message_token']
		self._user_id = request_dict['user_id']
		self._desc = request_dict['desc']
		return self

	@property
	def meesage_token(self):
		warnings.warn('Property `meesage_token` had typo and now is deprecated, please use `message_token` instead')
		return self._message_token

	@property
	def message_token(self):
		return self._message_token

	@property
	def user_id(self):
		return self._user_id

	@property
	def desc(self):
		return self._desc



from viberbot_prox.api.event_type import EventType
from viberbot_prox.api.viber_requests.viber_conversation_started_request import ViberConversationStartedRequest
from viberbot_prox.api.viber_requests.viber_delivered_request import ViberDeliveredRequest
from viberbot_prox.api.viber_requests.viber_failed_request import ViberFailedRequest
from viberbot_prox.api.viber_requests.viber_message_request import ViberMessageRequest
from viberbot_prox.api.viber_requests.viber_request import ViberRequest
from viberbot_prox.api.viber_requests.viber_seen_request import ViberSeenRequest
from viberbot_prox.api.viber_requests.viber_subscribed_request import ViberSubscribedRequest
from viberbot_prox.api.viber_requests.viber_unsubscribed_request import ViberUnsubscribedRequest

EVENT_TYPE_TO_CLASS = {
	EventType.MESSAGE: ViberMessageRequest,
	EventType.FAILED: ViberFailedRequest,
	EventType.CONVERSATION_STARTED: ViberConversationStartedRequest,
	EventType.DELIVERED: ViberDeliveredRequest,
	EventType.SEEN: ViberSeenRequest,
	EventType.SUBSCRIBED: ViberSubscribedRequest,
	EventType.UNSUBSCRIBED: ViberUnsubscribedRequest,
	EventType.WEBHOOK: ViberRequest
}


def create_request(request_dict):
	if 'event' not in request_dict:
		raise Exception("request is missing field 'event'")

	if request_dict['event'] not in EVENT_TYPE_TO_CLASS:
		raise Exception("event type '{0}' is not supported".format(request_dict['event']))

	return EVENT_TYPE_TO_CLASS[request_dict['event']]().from_dict(request_dict)


__all__ = [
	'ViberConversationStartedRequest', 'ViberDeliveredRequest', 'ViberFailedRequest', 'ViberMessageRequest',
	'ViberSeenRequest', 'ViberSubscribedRequest', 'ViberUnsubscribedRequest']



from viberbot_prox.api.event_type import EventType
from viberbot_prox.api.viber_requests.viber_request import ViberRequest


class ViberDeliveredRequest(ViberRequest):
	def __init__(self):
		super(ViberDeliveredRequest, self).__init__(EventType.DELIVERED)
		self._message_token = None
		self._user_id = None
		self._chat_id = None

	def from_dict(self, request_dict):
		super(ViberDeliveredRequest, self).from_dict(request_dict)
		self._message_token = request_dict['message_token']
		self._user_id = request_dict.get('user_id', None)
		self._chat_id = request_dict.get('chat_id', None)
		return self

	@property
	def message_token(self):
		return self._message_token

	@property
	def user_id(self):
		return self._user_id

	@property
	def chat_id(self):
		return self._chat_id



import hashlib
import hmac
import json
import logging

from viberbot_prox.api.consts import VIBER_BOT_API_URL, VIBER_BOT_USER_AGENT
from viberbot_prox.api.viber_requests import create_request
from viberbot_prox.api.api_request_sender import ApiRequestSender
from viberbot_prox.api.message_sender import MessageSender


class Api(object):
	def __init__(self, bot_configuration, proxies = None):
		self._logger = logging.getLogger('viber.bot.api')
		self._bot_configuration = bot_configuration
		self._request_sender = ApiRequestSender(
			self._logger,
			VIBER_BOT_API_URL,
			bot_configuration,
			VIBER_BOT_USER_AGENT,
			proxies,
		)
		self._message_sender = MessageSender(self._logger, self._request_sender, bot_configuration)

	@property
	def name(self):
		return self._bot_configuration.name

	@property
	def avatar(self):
		return self._bot_configuration.avatar

	def set_webhook(self, url, webhook_events=None, is_inline=False):
		self._logger.debug(u"setting webhook to url: {0}".format(url))
		return self._request_sender.set_webhook(url, webhook_events, is_inline)

	def unset_webhook(self):
		self._logger.debug("unsetting webhook")
		return self._request_sender.set_webhook('')

	def get_online(self, ids):
		return self._request_sender.get_online_status(ids)

	def get_user_details(self, user_id):
		return self._request_sender.get_user_details(user_id)

	def get_account_info(self):
		self._logger.debug("requesting account info")
		account_info = self._request_sender.get_account_info()
		self._logger.debug(u"received account info: {0}".format(account_info))
		return account_info

	def verify_signature(self, request_data, signature):
		return signature == self._calculate_message_signature(request_data)

	def parse_request(self, request_data):
		self._logger.debug("parsing request")
		request_dict = json.loads(request_data.decode() if isinstance(
			request_data, bytes) else request_data)
		request = create_request(request_dict)
		self._logger.debug(u"parsed request={0}".format(request))
		return request

	def send_messages(self, to, messages, chat_id=None):
		"""
		:param to: Viber user id
		:param messages: list of Message objects to be sent
		:param chat_id: Optional. String. Indicates that this is a message sent in inline conversation.
		:return: list of tokens of the sent messages
		"""
		self._logger.debug("going to send messages: {0}, to: {1}".format(messages, to))
		if not isinstance(messages, list):
			messages = [messages]

		sent_messages_tokens = []

		for message in messages:
			token = self._message_sender.send_message(
				to, self._bot_configuration.name, self._bot_configuration.avatar, message, chat_id)
			sent_messages_tokens.append(token)

		return sent_messages_tokens

	def post_messages_to_public_account(self, sender, messages):
		if not isinstance(messages, list):
			messages = [messages]

		sent_messages_tokens = []

		for message in messages:
			token = self._message_sender.post_to_public_account(
				sender, self._bot_configuration.name, self._bot_configuration.avatar, message)
			sent_messages_tokens.append(token)

		return sent_messages_tokens

	def _calculate_message_signature(self, message):
		return hmac.new(
			bytes(self._bot_configuration.auth_token.encode('ascii')),
			msg=message,
			digestmod=hashlib.sha256)\
			.hexdigest()



from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class URLMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, media=None, min_api_version=None):
		super(URLMessage, self).__init__(MessageType.URL, tracking_data, keyboard, min_api_version)
		self._media = media

	def to_dict(self):
		message_data = super(URLMessage, self).to_dict()
		message_data['media'] = self._media
		return message_data

	def from_dict(self, message_data):
		super(URLMessage, self).from_dict(message_data)
		if 'media' in message_data:
			self._media = message_data['media']
		return self

	@property
	def media(self):
		return self._media

	def validate(self):
		return super(URLMessage, self).validate() \
				and self._media is not None



class EventType(object):
	SEEN = 'seen'
	CONVERSATION_STARTED = 'conversation_started'
	DELIVERED = 'delivered'
	MESSAGE = 'message'
	SUBSCRIBED = 'subscribed'
	UNSUBSCRIBED = 'unsubscribed'
	FAILED = 'failed'
	WEBHOOK = 'webhook'



from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class FileMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, media=None, size=None, file_name=None, min_api_version=None):
		super(FileMessage, self).__init__(MessageType.FILE, tracking_data, keyboard, min_api_version)
		self._media = media
		self._size = size
		self._file_name = file_name

	def to_dict(self):
		message_data = super(FileMessage, self).to_dict()
		message_data['media'] = self._media
		message_data['size'] = self._size
		message_data['file_name'] = self._file_name
		return message_data

	def from_dict(self, message_data):
		super(FileMessage, self).from_dict(message_data)
		if 'media' in message_data:
			self._media = message_data['media']
		if 'size' in message_data:
			self._size = message_data['size']
		if 'file_name' in message_data:
			self._file_name = message_data['file_name']
		return self

	@property
	def media(self):
		return self._media

	@property
	def size(self):
		return self._size

	@property
	def file_name(self):
		return self._file_name

	def validate(self):
		return super(FileMessage, self).validate() \
				and self._media is not None \
				and self._size is not None \
				and self._file_name is not None



from viberbot_prox.api.event_type import EventType
from viberbot_prox.api.user_profile import UserProfile
from viberbot_prox.api.viber_requests.viber_request import ViberRequest


class ViberConversationStartedRequest(ViberRequest):
	def __init__(self):
		super(ViberConversationStartedRequest, self).__init__(EventType.CONVERSATION_STARTED)
		self._message_token = None
		self._type = None
		self._context = None
		self._user = None
		self._api_version = None
		self._subscribed = None

	def from_dict(self, request_dict):
		super(ViberConversationStartedRequest, self).from_dict(request_dict)
		self._message_token = request_dict['message_token']
		self._type = request_dict['type']
		if 'context' in request_dict:
			self._context = request_dict['context']
		self._user = UserProfile().from_dict(request_dict['user'])
		if 'api_version' in request_dict:
			self._api_version = request_dict['api_version']
		if 'subscribed' in request_dict:
			self._subscribed = request_dict['subscribed']
		return self

	@property
	def user(self):
		return self._user

	@property
	def type(self):
		return self._type

	@property
	def context(self):
		return self._context

	@property
	def message_token(self):
		return self._message_token

	@property
	def api_version(self):
		return self._api_version

	@property
	def subscribed(self):
		return self._subscribed



import requests
from requests import RequestException
import traceback
from viberbot_prox.api.consts import BOT_API_ENDPOINT
import json


class ApiRequestSender(object):
	def __init__(self, logger, viber_bot_api_url, bot_configuration, viber_bot_user_agent, proxies = None):
		self._logger = logger
		self._viber_bot_api_url = viber_bot_api_url
		self._bot_configuration = bot_configuration
		self._user_agent = viber_bot_user_agent
		self._proxies = proxies

	def set_webhook(self, url, webhook_events=None, is_inline=False):
		payload = {
			'auth_token': self._bot_configuration.auth_token,
			'url': url,
			'is_inline': is_inline
		}

		if webhook_events is not None:
			if not isinstance(webhook_events, list):
				webhook_events = [webhook_events]
			payload['event_types'] = webhook_events

		result = self.post_request(
			endpoint=BOT_API_ENDPOINT.SET_WEBHOOK,
			payload=json.dumps(payload))

		if not result['status'] == 0:
			raise Exception(u"failed with status: {0}, message: {1}".format(result['status'], result['status_message']))

		return result['event_types']

	def get_account_info(self):
		payload = {
			'auth_token': self._bot_configuration.auth_token
		}
		return self.post_request(
			endpoint=BOT_API_ENDPOINT.GET_ACCOUNT_INFO,
			payload=json.dumps(payload))

	def post_request(self, endpoint, payload):
		try:
			headers = requests.utils.default_headers()
			headers.update({
				'User-Agent': self._user_agent
			})
			response = requests.post(
				self._viber_bot_api_url + '/' + endpoint,
				data=payload,
				headers=headers,
				proxies=self._proxies,
			)
			response.raise_for_status()
			return json.loads(response.text)
		except RequestException as e:
			self._logger.error(
				u"failed to post request to endpoint={0}, with payload={1}. error is: {2}"
				.format(endpoint, payload, traceback.format_exc()))
			raise e
		except Exception as ex:
			self._logger.error(
				u"unexpected Exception while trying to post request. error is: {0}"
				.format(traceback.format_exc()))
			raise ex

	def get_online_status(self, ids=[]):
		if ids is None or not isinstance(ids, list) or len(ids) == 0:
			raise Exception(u"missing parameter ids, should be a list of viber memberIds")

		payload = {
			'auth_token': self._bot_configuration.auth_token,
			'ids': ids
		}
		result = self.post_request(
			endpoint=BOT_API_ENDPOINT.GET_ONLINE,
			payload=json.dumps(payload))

		if not result['status'] == 0:
			raise Exception(u"failed with status: {0}, message: {1}".format(result['status'], result['status_message']))

		return result['users']

	def get_user_details(self, user_id):
		if user_id is None:
			raise Exception(u"missing parameter id")

		payload = {
			'auth_token': self._bot_configuration.auth_token,
			'id': user_id
		}
		result = self.post_request(
			endpoint=BOT_API_ENDPOINT.GET_USER_DETAILS,
			payload=json.dumps(payload))

		if not result['status'] == 0:
			raise Exception(u"failed with status: {0}, message: {1}".format(result['status'], result['status_message']))

		return result['user']





from abc import abstractmethod


class Message(object):
	def __init__(self, tracking_data=None, keyboard=None, min_api_version=None, alt_text=None):
		self._tracking_data = tracking_data
		self._keyboard = keyboard
		self._min_api_version = min_api_version
		self._alt_text = alt_text

	@abstractmethod
	def to_dict(self):
		message_data = {}
		if self._tracking_data:
			message_data['tracking_data'] = self._tracking_data
		if self._keyboard:
			message_data['keyboard'] = self._keyboard
		if self._min_api_version:
			message_data['min_api_version'] = self._min_api_version
		if self._alt_text:
			message_data['alt_text'] = self._alt_text
		return message_data

	@abstractmethod
	def from_dict(self, message_data):
		if 'tracking_data' in message_data:
			self._tracking_data = message_data['tracking_data']
		if 'keyboard' in message_data:
			self._keyboard = message_data['keyboard']
		if 'min_api_version' in message_data:
			self._min_api_version = message_data['min_api_version']
		return self

	@property
	def keyboard(self):
		return self._keyboard

	@property
	def tracking_data(self):
		return self._tracking_data

	@property
	def min_api_version(self):
		return self._min_api_version

	@abstractmethod
	def validate(self):
		"""
		validates message has all the required fields before send
		"""
		pass



import json

from viberbot_prox.api.consts import BOT_API_ENDPOINT


class MessageSender(object):
	def __init__(self, logger, request_sender, bot_configuration):
		self._logger = logger
		self._request_sender = request_sender
		self._bot_configuration = bot_configuration

	def send_message(self, to, sender_name, sender_avatar, message, chat_id=None):
		if not message.validate():
			self._logger.error(u"failed validating message: {0}".format(message))
			raise Exception("failed validating message: {0}".format(message))

		payload = self._prepare_payload(
			message=message,
			receiver=to,
			sender_name=sender_name,
			sender_avatar=sender_avatar,
			chat_id=chat_id
		)

		self._logger.debug(u"going to send message: {0}".format(payload))

		return self._post_request(BOT_API_ENDPOINT.SEND_MESSAGE, payload)

	def post_to_public_account(self, sender, sender_name, sender_avatar, message):
		if not message.validate():
			self._logger.error(u"failed validating message: {0}".format(message))
			raise Exception("failed validating message: {0}".format(message))

		if sender is None:
			raise Exception(u"missing parameter sender")

		payload = self._prepare_payload(
			message=message,
			sender=sender,
			sender_name=sender_name,
			sender_avatar=sender_avatar
		)

		self._logger.debug(u"going to send message: {0}".format(payload))

		return self._post_request(BOT_API_ENDPOINT.POST, payload)

	def _post_request(self, endpoint, payload):
		result = self._request_sender.post_request(
			endpoint, json.dumps(payload))

		if not result['status'] == 0:
			raise Exception(u"failed with status: {0}, message: {1}".format(result['status'], result['status_message']))

		return result['message_token']

	def _prepare_payload(self, message, sender_name, sender_avatar, sender=None, receiver=None, chat_id=None):
		payload = message.to_dict()
		payload.update({
			'auth_token': self._bot_configuration.auth_token,
			'from': sender,
			'receiver': receiver,
			'sender': {
				'name': sender_name,
				'avatar': sender_avatar
			},
			"chat_id": chat_id
		})

		return self._remove_empty_fields(payload)

	def _remove_empty_fields(self, message):
		return {k: v for k, v in message.items() if v is not None}



class BotConfiguration(object):
	def __init__(self, auth_token, name, avatar):
		self._auth_token = auth_token
		self._name = name
		self._avatar = avatar

	@property
	def name(self):
		return self._name

	@property
	def avatar(self):
		return self._avatar

	@property
	def auth_token(self):
		return self._auth_token




class UserProfile(object):
	def __init__(self, name=None, avatar=None, user_id=None, country=None, language=None, api_version=None):
		self._name = name
		self._avatar = avatar
		self._id = user_id
		self._country = country
		self._language = language
		self._api_version = api_version

	@property
	def name(self):
		return self._name

	@property
	def avatar(self):
		return self._avatar

	@property
	def id(self):
		return self._id

	@property
	def country(self):
		return self._country

	@property
	def language(self):
		return self._language

	@property
	def api_version(self):
		return self._api_version

	def from_dict(self, user_dict):
		if 'name' in user_dict:
			self._name = user_dict['name']
		if 'avatar' in user_dict:
			self._avatar = user_dict['avatar']
		if 'id' in user_dict:
			self._id = user_dict['id']
		if 'country' in user_dict:
			self._country = user_dict['country']
		if 'language' in user_dict:
			self._language = user_dict['language']
		if 'api_version' in user_dict:
			self._api_version = user_dict['api_version']
		return self



from abc import abstractmethod
from viberbot_prox.api.messages.message import Message


class TypedMessage(Message):
	def __init__(self, message_type, tracking_data=None, keyboard=None, min_api_version=None, alt_text=None):
		super(TypedMessage, self).__init__(tracking_data, keyboard, min_api_version, alt_text)
		self._message_type = message_type

	@abstractmethod
	def to_dict(self):
		message_data = super(TypedMessage, self).to_dict()
		message_data['type'] = self._message_type
		return message_data

	@abstractmethod
	def from_dict(self, message_data):
		super(TypedMessage, self).from_dict(message_data)
		return self

	@abstractmethod
	def validate(self):
		"""
		validates message has all the required fields before send
		"""
		return self._message_type is not None




class LocationConsts(object):
	MAX_LONGITUDE = 180
	MIN_LONGITUDE = -180
	MAX_LATITUDE = 90
	MIN_LATITUDE = -90


class Location(object):
	def __init__(self, lat=None, lon=None):
		self._lat = lat
		self._lon = lon

	def from_dict(self, location):
		if 'lat' in location:
			self._lat = location['lat']
		if 'lon' in location:
			self._lon = location['lon']
		return self

	def to_dict(self):
		return {
			'lat': self._lat,
			'lon': self._lon
		}

	@property
	def latitude(self):
		return self._lat

	@property
	def longitude(self):
		return self._lon

	def validate(self):
		if self._lat is None or self._lon is None:
			return False
		if self._lat >= LocationConsts.MAX_LATITUDE or self._lat <= LocationConsts.MIN_LATITUDE:
			return False
		if self._lon >= LocationConsts.MAX_LONGITUDE or self._lon <= LocationConsts.MIN_LONGITUDE:
			return False
		return True

	def __eq__(self, other):
		return self._lat == other.latitude and self._lon == other.longitude



from viberbot_prox.api.messages.data_types.location import Location
from viberbot_prox.api.messages.typed_message import TypedMessage
from viberbot_prox.api.messages.message_type import MessageType


class LocationMessage(TypedMessage):
	def __init__(self, tracking_data=None, keyboard=None, location=None, min_api_version=None):
		super(LocationMessage, self).__init__(MessageType.LOCATION, tracking_data, keyboard, min_api_version)
		self._location = location

	def to_dict(self):
		message_data = super(LocationMessage, self).to_dict()
		if self._location is not None:
			message_data['location'] = self._location.to_dict()
		return message_data

	def from_dict(self, message_data):
		super(LocationMessage, self).from_dict(message_data)
		if 'location' in message_data:
			self._location = Location().from_dict(message_data['location'])
		return self

	@property
	def location(self):
		return self._location

	def validate(self):
		return super(LocationMessage, self).validate() \
				and self._location and self._location.validate()



class MessageType(object):
	RICH_MEDIA = 'rich_media'
	STICKER = 'sticker'
	URL = 'url'
	LOCATION = 'location'
	CONTACT = 'contact'
	FILE = 'file'
	TEXT = 'text'
	PICTURE = 'picture'
	VIDEO = 'video'
	KEYBOARD = 'keyboard'
